use crate::{
    entity::prelude::Account,
    errors::NoUserInfo,
    handlers::{markups::account_markup, utils::validate_field, MainDialogue},
};
use sea_orm::prelude::*;
use teloxide::{adaptors::Throttle, prelude::*};

/// Specifies the kind of checks to be run during the account name validation
#[derive(Clone, Copy)]
pub enum NameCheckKind {
    NewAccountName,
    MustExist,
}

/// Validates the account name
#[inline]
pub async fn check_name(
    bot: &Throttle<Bot>,
    msg: &Message,
    db: &DatabaseConnection,
    name: &str,
    check_kind: NameCheckKind,
) -> crate::Result<Option<Message>> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    match check_kind {
        NameCheckKind::MustExist => {
            if !Account::exists(user_id, name, db).await? {
                let msg = bot
                    .send_message(msg.chat.id, "Account doesn't exists. Try again")
                    .reply_markup(account_markup(user_id, db).await?)
                    .await?;
                return Ok(Some(msg));
            }
            Ok(None)
        }
        NameCheckKind::NewAccountName => {
            if Account::exists(user_id, name, db).await? {
                let msg = bot
                    .send_message(msg.chat.id, "Account alreay exists")
                    .await?;
                return Ok(Some(msg));
            }
            if !validate_field(name) {
                let msg = bot
                    .send_message(msg.chat.id, "Invalid account name. Try again")
                    .await?;
                return Ok(Some(msg));
            }
            Ok(None)
        }
    }
}

/// Function to handle GetAccountName state
pub async fn get_account_name(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    (next, check_kind): (super::PackagedHandler<String>, NameCheckKind),
) -> crate::Result<()> {
    super::generic::generic(
        bot,
        msg,
        db,
        dialogue,
        |bot, msg, db, name| Box::pin(check_name(bot, msg, db, name, check_kind)),
        next,
    )
    .await
}
